@@ -27,8 +27,8 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
-static char *DownloadSuffix=NULL;
-static char **DownloadSuffixIndex=NULL;
+/*@null@*/static char *DownloadSuffix=NULL;
+/*@null@*/static char **DownloadSuffixIndex=NULL;
 static int NDownloadSuffix=0;
 
 void download_report(void)
@@ -36,11 +36,10 @@ void download_report(void)
 
    FILE *fp_in = NULL, *fp_ou = NULL;
 
-   char url[2*MAXLEN];
+   char *buf;
+   char *url;
    char report_in[MAXLEN];
-   char wdirname[MAXLEN];
    char report[MAXLEN];
-   char period[100];
    char ip[MAXLEN];
    char oip[MAXLEN];
    char user[MAXLEN];
@@ -48,135 +47,125 @@ void download_report(void)
    char ouser2[MAXLEN];
    char data[15];
    char hora[15];
-   char *str;
    int  z=0;
    int  count=0;
    int i;
+   int day,month,year;
+   bool new_user;
    struct getwordstruct gwarea;
+   longline line;
+   struct userinfostruct *uinfo;
+   struct tm t;
 
    ouser[0]='\0';
    ouser2[0]='\0';
 
-   sprintf(report_in,"%s/sarg/download.log",TempDir);
+   snprintf(report_in,sizeof(report_in),"%s/sarg/download.log",TempDir);
    if(access(report_in, R_OK) != 0)
       return;
 
-   strcpy(wdirname,dirname);
-   sprintf(report,"%s/download.html",wdirname);
-   strcat(wdirname,"/");
-   strcat(wdirname,"sarg-period");
-
-   if ((fp_in = fopen(wdirname, "r")) == 0) {
-      fprintf(stderr, "SARG: (download) %s: %s\n",text[45],wdirname);
-      exit(1);
-   }
-
-   if (!fgets(period,sizeof(period),fp_in)) {
-      fprintf(stderr,"SARG: (download) read error in %s\n",wdirname);
-      exit(1);
-   }
-   fclose(fp_in);
+   snprintf(report,sizeof(report),"%s/download.html",outdirname);
 
    if((fp_in=MY_FOPEN(report_in,"r"))==NULL) {
-     fprintf(stderr, "SARG: (download) %s: %s\n",text[8],report_in);
-     exit(1);
+      debuga(_("(download) Cannot open log file %s\n"),report_in);
+      exit(EXIT_FAILURE);
    }
 
    if((fp_ou=MY_FOPEN(report,"w"))==NULL) {
-     fprintf(stderr, "SARG: (download) %s: %s\n",text[8],report);
-     exit(1);
+      debuga(_("(download) Cannot open log file %s\n"),report);
+      exit(EXIT_FAILURE);
    }
 
-   fprintf(fp_ou, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
-   css(fp_ou);
-   fputs("</head>\n",fp_ou);
+   write_html_header(fp_ou,(IndexTree == INDEX_TREE_DATE) ? 3 : 1,_("Downloads"));
+   fputs("<tr><td class=\"header_c\">",fp_ou);
+   fprintf(fp_ou,_("Period: %s"),period.html);
+   fputs("</td></tr>\n",fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"header_c\">%s</th></tr>\n",_("Downloads"));
+   close_html_header(fp_ou);
 
-   fprintf(fp_ou,"<body bgcolor=\"%s\" text=\"%s\" background=\"%s\">\n",BgColor,TxColor,BgImage);
+   fputs("<div class=\"report\"><table cellpadding=\"0\" cellspacing=\"2\">\n",fp_ou);
+   fprintf(fp_ou,"<tr><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th></tr>\n",_("USERID"),_("IP/NAME"),_("DATE/TIME"),_("ACCESSED SITE"));
 
-   write_logo_image(fp_ou);
+   if ((line=longline_create())==NULL) {
+      debuga(_("Not enough memory to read the downloaded files\n"));
+      exit(EXIT_FAILURE);
+   }
 
-   if(strcmp(IndexTree,"date") == 0)
-      show_sarg(fp_ou, "../../..");
-   else
-      show_sarg(fp_ou, "..");
-
-   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"0\">\n",fp_ou);
-   fprintf(fp_ou,"<tr><th class=\"title\">%s</th></tr>\n",Title);
-
-   fprintf(fp_ou,"<tr><td class=\"header\">%s: %s</td></tr>\n",text[89],period);
-   fprintf(fp_ou,"<tr><th class=\"header3\">%s</th></tr>\n",text[125]);
-   fputs("</table></div>\n",fp_ou);
-
-   fputs("<div align=\"center\"><table cellpadding=\"0\" cellspacing=\"2\">\n",fp_ou);
-   fputs("<tr><td></td></tr>\n",fp_ou);
-   fprintf(fp_ou,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91]);
-
-   while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
+   while((buf=longline_read(fp_in,line))!=NULL) {
       getword_start(&gwarea,buf);
       if (getword(data,sizeof(data),&gwarea,'\t')<0 || getword(hora,sizeof(hora),&gwarea,'\t')<0 ||
-          getword(user,sizeof(user),&gwarea,'\t')<0 || getword(ip,sizeof(ip),&gwarea,'\t')<0 ||
-          getword(url,sizeof(url),&gwarea,'\t')<0) {
-         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",report_in);
-         exit(1);
+          getword(user,sizeof(user),&gwarea,'\t')<0 || getword(ip,sizeof(ip),&gwarea,'\t')<0) {
+         debuga(_("There is a broken record or garbage in file %s\n"),report_in);
+         exit(EXIT_FAILURE);
       }
-
-      if((str=(char *) strstr(user, "_")) != (char *) NULL ) {
-         if((str=(char *) strstr(str+1, "_")) != (char *) NULL )
-            fixip(user);
+      if (getword_ptr(buf,&url,&gwarea,'\t')<0) {
+         debuga(_("There is a broken url in file %s\n"),report_in);
+         exit(EXIT_FAILURE);
       }
+      if (sscanf(data,"%d/%d/%d",&day,&month,&year)!=3) continue;
+      computedate(year,month,day,&t);
+      strftime(data,sizeof(data),"%x",&t);
 
-      if(Ip2Name)
-         ip2name(ip,sizeof(ip));
-
+      uinfo=userinfo_find_from_id(user);
+      if (!uinfo) {
+         debuga(_("Unknown user ID %s in file %s\n"),user,report_in);
+         exit(EXIT_FAILURE);
+      }
+      new_user=false;
       if(!z) {
          strcpy(ouser,user);
          strcpy(oip,ip);
          z++;
+         new_user=true;
       } else {
-         if(strcmp(ouser,user) == 0)
-            user[0]='\0';
-         if(user[0] != '\0')
+         if(strcmp(ouser,user) != 0) {
             strcpy(ouser,user);
-         if(strcmp(oip,ip) == 0)
-            ip[0]='\0';
-         if(ip[0] != '\0')
+            new_user=true;
+         }
+         if(strcmp(oip,ip) != 0) {
             strcpy(oip,ip);
-      }
-
-      get_usertab_name(user,name,sizeof(name));
-
-      if(dotinuser && strchr(name,'_')) {
-         subs(name,sizeof(name),"_",".");
+            new_user=true;
+         }
       }
 
       if(DownloadReportLimit) {
-         if(strcmp(ouser2,name) == 0) {
+         if(strcmp(ouser2,uinfo->label) == 0) {
             count++;
          } else {
             count=1;
-            strcpy(ouser2,name);
+            strcpy(ouser2,uinfo->label);
          }
          if(count >= DownloadReportLimit)
             continue;
       }
 
-      for (i=strlen(url)-1 ; i>=0 && (unsigned char)url[i]<' ' ; i--) url[i]='\0';
+      for (i=strlen(url)-1 ; i>=0 && (unsigned char)url[i]<' ' ; i--) url[i]=0;
 
-      fprintf(fp_ou,"<tr><td class=\"data\">%s</td><td class=\"data\">%s</td><td class=\"data\">%s-%s</td><td class=\"data2\">",name,ip,data,hora);
-      if(BlockIt[0]!='\0')
-         fprintf(fp_ou,"<a href=\"%s%s?url=%s\"><img src=\"%s/sarg-squidguard-block.png\" border=\"0\"></a>&nbsp;",wwwDocumentRoot,BlockIt,url,ImageFile);
-      fprintf(fp_ou,"<a href=\"%s\">",url);
-      output_html_string(fp_ou,url);
+      fputs("<tr>",fp_ou);
+      if (new_user)
+         fprintf(fp_ou,"<td class=\"data\">%s</td><td class=\"data\">%s</td>",uinfo->label,ip);
+      else
+         fputs("<td class=\"data\"></td><td class=\"data\"></td>",fp_ou);
+      fprintf(fp_ou,"<td class=\"data\">%s-%s</td><td class=\"data2\">",data,hora);
+      if(BlockIt[0]!='\0') {
+         fprintf(fp_ou,"<a href=\"%s%s?url=\"",wwwDocumentRoot,BlockIt);
+         output_html_url(fp_ou,url);
+         fprintf(fp_ou,"\"><img src=\"%s/sarg-squidguard-block.png\"></a>&nbsp;",ImageFile);
+      }
+      fputs("<a href=\"http://",fp_ou);
+      output_html_url(fp_ou,url);
+      fputs("\">http://",fp_ou);
+      output_html_string(fp_ou,url,100);
       fputs("</a></td></tr>\n",fp_ou);
    }
-
-   fputs("</table>\n",fp_ou);
-
-   show_info(fp_ou);
-   fputs("</body>\n</html>\n",fp_ou);
-
    fclose(fp_in);
-   fclose(fp_ou);
+   longline_destroy(&line);
+
+   fputs("</table></div>\n",fp_ou);
+   if (write_html_trailer(fp_ou)<0)
+      debuga(_("Write error in file %s\n"),report);
+   if (fclose(fp_ou)==EOF)
+      debuga(_("Failed to close file %s - %s\n"),report,strerror(errno));
 
    unlink(report_in);
 
@@ -206,16 +195,16 @@ void set_download_suffix(const char *list)
 
    DownloadSuffix=strdup(list);
    if (!DownloadSuffix) {
-      fprintf(stderr,"SARG: Download suffix list too long\n");
-      exit(1);
+      debuga(_("Download suffix list too long\n"));
+      exit(EXIT_FAILURE);
    }
    j = 1;
    for (i=0 ; list[i] ; i++)
       if (list[i] == ',') j++;
    DownloadSuffixIndex=malloc(j*sizeof(char *));
    if (!DownloadSuffixIndex) {
-      fprintf(stderr,"SARG: Too many download suffixes\n");
-      exit(1);
+      debuga(_("Too many download suffixes\n"));
+      exit(EXIT_FAILURE);
    }
 
    str = DownloadSuffix;
@@ -248,7 +237,7 @@ void set_download_suffix(const char *list)
    }
 }
 
-int is_download_suffix(const char *url)
+bool is_download_suffix(const char *url)
 {
    int urllen;
    int i;
@@ -257,17 +246,17 @@ int is_download_suffix(const char *url)
    int cmp;
    const int max_suffix=10;
 
-   if (DownloadSuffix == NULL || NDownloadSuffix == 0) return(0);
+   if (DownloadSuffix == NULL || NDownloadSuffix == 0) return(false);
 
    urllen=strlen(url)-1;
-   if (urllen<=0) return(0);
-   if (url[urllen] == '.') return(0); //reject a single trailing dot
+   if (urllen<=0) return(false);
+   if (url[urllen] == '.') return(false); //reject a single trailing dot
    for (i=0 ; i<urllen && (url[i]!='/' || url[i+1]=='/') && url[i]!='?' ; i++);
-   if (i>=urllen) return(0); // url is a hostname without any path or file to download
+   if (i>=urllen) return(false); // url is a hostname without any path or file to download
 
    for (i=0 ; i<=max_suffix && i<urllen && url[urllen-i]!='.' ; i++)
-      if (url[urllen-i] == '/' || url[urllen-i] == '?') return(0);
-   if (i>max_suffix || i>=urllen) return(0);
+      if (url[urllen-i] == '/' || url[urllen-i] == '?') return(false);
+   if (i>max_suffix || i>=urllen) return(false);
 
    suffix=url+urllen-i+1;
    down=0;
@@ -275,12 +264,12 @@ int is_download_suffix(const char *url)
    while (down<=up) {
       center=(down+up)/2;
       cmp=strcasecmp(suffix,DownloadSuffixIndex[center]);
-      if (cmp == 0) return(1);
+      if (cmp == 0) return(true);
       if (cmp < 0)
          up = center-1;
       else
          down = center+1;
    }
-   return(0);
+   return(false);
 }
 
