@@ -27,8 +27,127 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
+#define SET_LIST(list) list,sizeof(list)/sizeof(list[0])
+
 extern numlist hours, weekdays;
 
+struct param_list
+{
+   //! The name of the value of the parameter.
+   const char *name;
+   //! The bit to set if the value is found.
+   unsigned long int value;
+   //! The value is invalid if any bit of this mask is set in the parameter.
+   unsigned long int exclude;
+};
+
+static struct param_list report_type_values[]=
+{
+   {"users_sites",REPORT_TYPE_USERS_SITES,0},
+   {"topusers",REPORT_TYPE_TOPUSERS,0},
+   {"topsites",REPORT_TYPE_TOPSITES,0},
+   {"sites_users",REPORT_TYPE_SITES_USERS,0},
+   {"date_time",REPORT_TYPE_DATE_TIME,0},
+   {"denied",REPORT_TYPE_DENIED,0},
+   {"auth_failures",REPORT_TYPE_AUTH_FAILURES,0},
+   {"site_user_time_date",REPORT_TYPE_SITE_USER_TIME_DATE,0},
+   {"downloads",REPORT_TYPE_DOWNLOADS,0},
+};
+
+static struct param_list data_field_values[]=
+{
+   {"user",DATA_FIELD_USER,0},
+   {"date",DATA_FIELD_DATE,0},
+   {"time",DATA_FIELD_TIME,0},
+   {"url",DATA_FIELD_URL,0},
+   {"connect",DATA_FIELD_CONNECT,0},
+   {"bytes",DATA_FIELD_BYTES,0},
+   {"in_cache",DATA_FIELD_IN_CACHE,0},
+   {"out_cache",DATA_FIELD_OUT_CACHE,0},
+   {"elapsed",DATA_FIELD_ELAPSED,0},
+};
+
+static struct param_list topuserfields_values[]=
+{
+   {"NUM",TOPUSERFIELDS_NUM,0},
+   {"DATE_TIME",TOPUSERFIELDS_DATE_TIME,0},
+   {"USERID",TOPUSERFIELDS_USERID,0},
+   {"CONNECT",TOPUSERFIELDS_CONNECT,0},
+   {"BYTES",TOPUSERFIELDS_BYTES,0},
+   {"%BYTES",TOPUSERFIELDS_SETYB,0},
+   {"SETYB",TOPUSERFIELDS_SETYB,0},
+   {"IN-CACHE-OUT",TOPUSERFIELDS_IN_CACHE_OUT,0},
+   {"USED_TIME",TOPUSERFIELDS_USED_TIME,0},
+   {"MILISEC",TOPUSERFIELDS_MILISEC,0},
+   {"%TIME",TOPUSERFIELDS_PTIME,0},
+   {"TOTAL",TOPUSERFIELDS_TOTAL,0},
+   {"AVERAGE",TOPUSERFIELDS_AVERAGE,0},
+};
+
+static struct param_list userreportfields_values[]=
+{
+   {"CONNECT",USERREPORTFIELDS_CONNECT,0},
+   {"BYTES",USERREPORTFIELDS_BYTES,0},
+   {"%BYTES",USERREPORTFIELDS_SETYB,0},
+   {"SETYB",USERREPORTFIELDS_SETYB,0},
+   {"IN-CACHE-OUT",USERREPORTFIELDS_IN_CACHE_OUT,0},
+   {"USED_TIME",USERREPORTFIELDS_USED_TIME,0},
+   {"MILISEC",USERREPORTFIELDS_MILISEC,0},
+   {"%TIME",USERREPORTFIELDS_PTIME,0},
+   {"TOTAL",USERREPORTFIELDS_TOTAL,0},
+   {"AVERAGE",USERREPORTFIELDS_AVERAGE,0},
+};
+
+static struct param_list index_values[]=
+{
+   {"yes",INDEX_YES,~INDEX_YES},
+   {"no",INDEX_NO,~INDEX_NO},
+   {"only",INDEX_ONLY,~INDEX_ONLY},
+};
+
+static struct param_list index_tree_values[]=
+{
+   {"date",INDEX_TREE_DATE,~INDEX_TREE_DATE},
+   {"file",INDEX_TREE_FILE,~INDEX_TREE_FILE},
+};
+
+static struct param_list ntml_userformat_values[]=
+{
+   {"user",NTLMUSERFORMAT_USER,~NTLMUSERFORMAT_USER},
+   {"domainname+username",NTLMUSERFORMAT_DOMAINUSER,~NTLMUSERFORMAT_DOMAINUSER},
+};
+
+static struct param_list recnouser_values[]=
+{
+   {"ip",RECORDWITHOUTUSER_IP,~RECORDWITHOUTUSER_IP},
+   {"ignore",RECORDWITHOUTUSER_IGNORE,~RECORDWITHOUTUSER_IGNORE},
+   {"everybody",RECORDWITHOUTUSER_EVERYBODY,~RECORDWITHOUTUSER_EVERYBODY},
+};
+
+static struct param_list datafileurl_values[]=
+{
+   {"ip",DATAFILEURL_IP,~DATAFILEURL_IP},
+   {"name",DATAFILEURL_NAME,~DATAFILEURL_NAME},
+};
+
+static struct param_list displayvalue_values[]=
+{
+   {"bytes",DISPLAY_BYTES,~DISPLAY_BYTES},
+   {"abbreviation",DISPLAY_ABBREV,~DISPLAY_ABBREV},
+};
+
+static struct param_list datetime_values[]=
+{
+   {"elap",DATETIME_ELAP,0},
+   {"bytes",DATETIME_BYTE,0},
+};
+
+static struct param_list realtime_unauth_values[]=
+{
+   {"show",REALTIME_UNAUTH_REC_SHOW,~REALTIME_UNAUTH_REC_SHOW},
+   {"ignore",REALTIME_UNAUTH_REC_IGNORE,~REALTIME_UNAUTH_REC_IGNORE},
+};
+
 static int is_param(const char *param,const char *buf)
 {
    int plen;
@@ -51,8 +170,8 @@ static int getparam_string(const char *param,char *buf,char *value,int value_siz
    while (*buf && (unsigned char)*buf<=' ') buf++;
 
    if (strlen(buf)>=value_size) {
-      printf("SARG: Maybe you have a broken record or garbage in %s parameter.\n",param);
-      exit(1);
+      debuga(_("The string value of parameter \"%s\" is too long\n"),param);
+      exit(EXIT_FAILURE);
    }
    strcpy(value,buf);
    fixnone(value);
@@ -71,8 +190,8 @@ static int getparam_quoted(const char *param,char *buf,char *value,int value_siz
    while (*buf && (unsigned char)*buf<=' ') buf++;
 
    if (*buf != '\"') {
-      printf("SARG: %s %s.\n",text[139],param);
-      exit(1);
+      debuga(_("Missing double quote after parameter \"%s\"\n"),param);
+      exit(EXIT_FAILURE);
    }
    buf++;
 
@@ -83,8 +202,8 @@ static int getparam_quoted(const char *param,char *buf,char *value,int value_siz
    value[i]='\0';
 
    if (*buf != '\"') {
-      printf("SARG: Missing double quote after parameter %s or value is more than %d bytes long.\n",param,value_size);
-      exit(1);
+      debuga(_("Missing double quote after parameter \"%s\" or value is more than %d bytes long\n"),param,value_size);
+      exit(EXIT_FAILURE);
    }
    fixnone(value);
    return(1);
@@ -104,12 +223,12 @@ static int getparam_2words(const char *param,char *buf,char *word1,int word1_siz
    for (i=0 ; i<word1_size && *buf && (unsigned char)*buf>' ' ; i++)
       word1[i]=*buf++;
    if (i>=word1_size) {
-      printf("SARG: The first word of parameter %s is more than %d bytes long.\n",param,word1_size-1);
-      exit(1);
+      debuga(_("The first word of parameter \"%s\" is more than %d bytes long\n"),param,word1_size-1);
+      exit(EXIT_FAILURE);
    }
    if (*buf!=' ') {
-      printf("SARG: Missing second word for parameter %s.\n",param);
-      exit(1);
+      debuga(_("Missing second word for parameter \"%s\"\n"),param);
+      exit(EXIT_FAILURE);
    }
    word1[i]=0;
 
@@ -118,8 +237,8 @@ static int getparam_2words(const char *param,char *buf,char *word1,int word1_siz
    for (i=0 ; i<word2_size && *buf && (unsigned char)*buf>' ' ; i++)
       word2[i]=*buf++;
    if (i>=word2_size) {
-      printf("SARG: The second word of parameter %s is more than %d bytes long.\n",param,word2_size-1);
-      exit(1);
+      debuga(_("The second word of parameter \"%s\" is more than %d bytes long\n"),param,word2_size-1);
+      exit(EXIT_FAILURE);
    }
    word2[i]=0;
 
@@ -141,13 +260,13 @@ static int getparam_int(const char *param,char *buf,int *value)
 
    next=0;
    if (sscanf(buf,"%d%n",value,&next) != 1 || (unsigned char)buf[next] > ' ') {
-      printf("SARG: Maybe you have a broken record or garbage in %s parameter.\n",param);
-      exit(1);
+      debuga(_("The integer value of parameter \"%s\" is invalid\n"),param);
+      exit(EXIT_FAILURE);
    }
    return(1);
 }
 
-static int getparam_bool(const char *param,char *buf,int *value)
+static int getparam_bool(const char *param,char *buf,bool *value)
 {
    int plen;
    int i;
@@ -159,11 +278,11 @@ static int getparam_bool(const char *param,char *buf,int *value)
    if ((unsigned char)*buf>' ') return(0);
    while (*buf && (unsigned char)*buf<=' ') buf++;
 
-   *value=0;
+   *value=false;
    for ( ; *bool_str ; bool_str+=i) {
       for (i=0 ; bool_str[i] && bool_str[i]!=',' ; i++);
       if (strncasecmp(bool_str,buf,i)==0) {
-         *value=1;
+         *value=true;
          break;
       }
       if (bool_str[i]==',') i++;
@@ -171,9 +290,44 @@ static int getparam_bool(const char *param,char *buf,int *value)
    return(1);
 }
 
+static int getparam_list(const char *param,struct param_list *options,int noptions,char *buf,unsigned long int *value)
+{
+   int plen;
+   char *str;
+   int i;
+
+   plen=strlen(param);
+   if (strncmp(buf,param,plen) != 0) return(0);
+   buf+=plen;
+   if ((unsigned char)*buf>' ') return(0);
+   while (*buf && (unsigned char)*buf<=' ') buf++;
+
+   *value=0UL;
+   while (*buf) {
+      str=buf;
+      while (*str && (unsigned char)*str>' ' && *str!=';') str++;
+      if (*str) {
+         *str++='\0';
+         while (*str && ((unsigned char)*str<=' ' || *str==';')) str++;
+      }
+      for (i=0 ; i<noptions && strcasecmp(buf,options[i].name) ; i++);
+      if (i>=noptions) {
+         debuga(_("Unknown value \"%s\" for parameter \"%s\"\n"),buf,param);
+         exit(EXIT_FAILURE);
+      }
+      if ((*value & options[i].exclude)!=0) {
+         debuga(_("Value \"%s\" conflicts with other selected values for parameter \"%s\"\n"),buf,param);
+         exit(EXIT_FAILURE);
+      }
+      *value|=options[i].value;
+      buf=str;
+   }
+   return(1);
+}
+
 static void parmtest(char *buf)
 {
-   char wbuf[50];
+   char wbuf[2048];
    struct getwordstruct gwarea;
 
    while (*buf && (unsigned char)*buf<=' ') buf++;
@@ -182,7 +336,7 @@ static void parmtest(char *buf)
       return;
 
    if(debugz)
-      printf("SARG: TAG: %s\n",buf);
+      printf(_("SARG: TAG: %s\n"),buf);
 
    if (getparam_string("background_color",buf,BgColor,sizeof(BgColor))>0) return;
 
@@ -229,8 +383,8 @@ static void parmtest(char *buf)
    if (is_param("date_format",buf)) {
       getword_start(&gwarea,buf);
       if (getword_multisep(wbuf,sizeof(wbuf),&gwarea,' ')<0) {
-         printf("SARG: Maybe you have a broken record or garbage in date_format parameter.\n");
-         exit(1);
+         debuga(_("Maybe you have a broken record or garbage in \"date_format\" parameter\n"));
+         exit(EXIT_FAILURE);
       }
       strncpy(DateFormat,gwarea.current,1);
       fixnone(DateFormat);
@@ -239,14 +393,14 @@ static void parmtest(char *buf)
 
    if (is_param("hours",buf)) {
       if( getnumlist( buf, &hours, 24, 24 ) ) {
-         fprintf( stderr, "Error: Invalid syntax in hours tag!\n" );
+         debuga(_("Error: Invalid syntax in hours tag!\n"));
          exit( 1 );
       }
    }
 
    if (is_param("weekdays",buf)) {
       if( getnumlist( buf, &weekdays, 7, 7 ) ) {
-         fprintf( stderr, "Error: Invalid syntax in weekdays tag!\n" );
+         debuga(_("Error: Invalid syntax in weekdays tag!\n"));
          exit( 1 );
       }
    }
@@ -258,15 +412,27 @@ static void parmtest(char *buf)
    if (is_param("access_log",buf)>0) {
       if (AccessLogFromCmdLine==0) {
          if (NAccessLog>=MAXLOGS) {
-            fprintf(stderr,"SARG: Too many log files.\n");
-            exit(1);
+            debuga(_("Too many log files in configuration file\n"));
+            exit(EXIT_FAILURE);
          }
-         getparam_string("access_log",buf,AccessLog[NAccessLog],MAXLEN);
+         getparam_string("access_log",buf,AccessLog[NAccessLog],MAX_LOG_FILELEN);
          NAccessLog++;
       }
       return;
    }
 
+   if (is_param("redirector_log",buf)>0) {
+      if (RedirectorLogFromCmdLine==0) {
+         if (NRedirectorLogs>=MAX_REDIRECTOR_LOGS) {
+            debuga(_("Too many redirector log files in configuration file\n"));
+            exit(EXIT_FAILURE);
+         }
+         getparam_string("redirector_log",buf,RedirectorLogs[NRedirectorLogs],MAX_REDIRECTOR_FILELEN);
+         NRedirectorLogs++;
+      }
+      return;
+   }
+
    if (getparam_string("useragent_log",buf,UserAgentLog,sizeof(UserAgentLog))>0) return;
 
    if (getparam_string("exclude_hosts",buf,ExcludeHosts,sizeof(ExcludeHosts))>0) return;
@@ -279,7 +445,7 @@ static void parmtest(char *buf)
 
    if (getparam_string("temporary_dir",buf,TempDir,sizeof(TempDir))>0) return;
 
-   if (getparam_string("report_type",buf,ReportType,sizeof(ReportType))>0) return;
+   if (getparam_list("report_type",SET_LIST(report_type_values),buf,&ReportType)>0) return;
 
    if (getparam_string("output_dir",buf,OutputDir,sizeof(OutputDir))>0) return;
 
@@ -296,13 +462,13 @@ static void parmtest(char *buf)
 
    if (getparam_string("replace_index",buf,ReplaceIndex,sizeof(ReplaceIndex))>0) return;
 
-   if (getparam_string("index_tree",buf,IndexTree,sizeof(IndexTree))>0) return;
+   if (getparam_list("index_tree",SET_LIST(index_tree_values),buf,&IndexTree)>0) return;
 
-   if (getparam_string("index",buf,Index,sizeof(Index))>0) return;
+   if (getparam_list("index",SET_LIST(index_values),buf,&Index)>0) return;
 
    if (getparam_bool("overwrite_report",buf,&OverwriteReport)>0) return;
 
-   if (getparam_string("records_without_userid",buf,RecordsWithoutUser,sizeof(RecordsWithoutUser))>0) return;
+   if (getparam_list("records_without_userid",SET_LIST(recnouser_values),buf,&RecordsWithoutUser)>0) return;
 
    if (getparam_bool("use_comma",buf,&UseComma)>0) return;
 
@@ -320,13 +486,11 @@ static void parmtest(char *buf)
 
    if (getparam_bool("long_url",buf,&LongUrl)>0) return;
 
-   if (getparam_string("language",buf,language,sizeof(language))>0) return;
-
    if (getparam_string("dansguardian_conf",buf,DansGuardianConf,sizeof(DansGuardianConf))>0) return;
 
    if (getparam_string("squidguard_conf",buf,SquidGuardConf,sizeof(SquidGuardConf))>0) return;
 
-   if (getparam_string("date_time_by",buf,datetimeby,sizeof(datetimeby))>0) return;
+   if (getparam_list("date_time_by",SET_LIST(datetime_values),buf,&datetimeby)>0) return;
 
    if (getparam_string("charset",buf,CharSet,sizeof(CharSet))>0) {
       ccharset(CharSet);
@@ -353,25 +517,25 @@ static void parmtest(char *buf)
 
    if (getparam_bool("show_read_statistics",buf,&ShowReadStatistics)>0) return;
 
-   if (getparam_string("topuser_fields",buf,TopUserFields,sizeof(TopUserFields))>0) return;
+   if (getparam_list("topuser_fields",SET_LIST(topuserfields_values),buf,&TopUserFields)>0) return;
 
    if (getparam_bool("bytes_in_sites_users_report",buf,&BytesInSitesUsersReport)>0) return;
 
-   if (getparam_string("user_report_fields",buf,UserReportFields,sizeof(UserReportFields))>0) return;
+   if (getparam_list("user_report_fields",SET_LIST(userreportfields_values),buf,&UserReportFields)>0) return;
 
    if (getparam_string("datafile",buf,DataFile,sizeof(DataFile))>0) return;
 
    if (getparam_quoted("datafile_delimiter",buf,DataFileDelimiter,sizeof(DataFileDelimiter))>0) return;
 
-   if (getparam_string("datafile_fields",buf,DataFileFields,sizeof(DataFileFields))>0) return;
+   if (getparam_list("datafile_fields",SET_LIST(data_field_values),buf,&DataFileFields)>0) return;
 
-   if (getparam_string("datafile_url",buf,DataFileUrl,sizeof(DataFileUrl))>0) return;
+   if (getparam_list("datafile_url",SET_LIST(datafileurl_values),buf,&DataFileUrl)>0) return;
 
    if (getparam_string("parsed_output_log",buf,ParsedOutputLog,sizeof(ParsedOutputLog))>0) return;
 
    if (getparam_string("parsed_output_log_compress",buf,ParsedOutputLogCompress,sizeof(ParsedOutputLogCompress))>0) return;
 
-   if (getparam_string("displayed_values",buf,DisplayedValues,sizeof(DisplayedValues))>0) return;
+   if (getparam_list("displayed_values",SET_LIST(displayvalue_values),buf,&DisplayedValues)>0) return;
 
    if (getparam_int("authfail_report_limit",buf,&AuthfailReportLimit)>0) return;
 
@@ -395,13 +559,24 @@ static void parmtest(char *buf)
 
    if (getparam_bool("user_authentication",buf,&UserAuthentication)>0) return;
 
-   if (getparam_string("AuthUserFile",buf,AuthUserFile,sizeof(AuthUserFile))>0) return;
+   if (getparam_string("AuthUserTemplateFile",buf,wbuf,sizeof(wbuf))>0) {
+      char dir[MAXLEN];
 
-   if (getparam_string("AuthName",buf,AuthName,sizeof(AuthName))>0) return;
-
-   if (getparam_string("AuthType",buf,AuthType,sizeof(AuthType))>0) return;
-
-   if (getparam_string("Require",buf,Require,sizeof(Require))>0) return;
+      if (is_absolute(wbuf)) {
+         if (strlen(wbuf)>=sizeof(AuthUserTemplateFile)) {
+            debuga(_("Template file name is too long in parameter \"AuthUserTemplateFile\"\n"));
+            exit(EXIT_FAILURE);
+         }
+         strcpy(AuthUserTemplateFile,wbuf);
+      } else {
+         strcpy(dir,ConfigFile);
+         if (snprintf(AuthUserTemplateFile,sizeof(AuthUserTemplateFile),"%s/%s",dirname(dir),wbuf)>=sizeof(AuthUserTemplateFile)) {
+            debuga(_("Template file name is too long in parameter \"AuthUserTemplateFile\"\n"));
+            exit(EXIT_FAILURE);
+         }
+      }
+      return;
+   }
 
    if (is_param("download_suffix",buf)) {
       char warea[MAXLEN];
@@ -415,47 +590,85 @@ static void parmtest(char *buf)
 
    if (getparam_string("graph_days_bytes_bar_color",buf,GraphDaysBytesBarColor,sizeof(GraphDaysBytesBarColor))>0) return;
 
-   if (getparam_string("squidguard_log_format",buf,SquidGuardLogFormat,sizeof(SquidGuardLogFormat))>0) return;
+   if (getparam_string("redirector_log_format",buf,RedirectorLogFormat,sizeof(RedirectorLogFormat))>0) return;
+   if (getparam_string("squidguard_log_format",buf,RedirectorLogFormat,sizeof(RedirectorLogFormat))>0) {
+      debuga(_("squidguard_log_format is deprecated and has been replaced by redirector_log_format. Please update your configuration file.\n"));
+      return;
+   }
 
-   if (getparam_bool("squidguard_ignore_date",buf,&SquidguardIgnoreDate)>0) return;
+   if (getparam_bool("redirector_filter_out_date",buf,&RedirectorFilterOutDate)>0) return;
+   if (getparam_bool("redirector_ignore_date",buf,&RedirectorFilterOutDate)>0) {
+      /*
+      Due to an old bug in sarg before version 2.3, the option was having the opposite action than implied by the name.
+      */
+      debuga(_("redirector_ignore_date is deprecated and has been replaced by redirector_filter_out_date that does the action implied by its name as opposed to redirector_ignore_date. Please update your configuration file.\n"));
+      RedirectorFilterOutDate=!RedirectorFilterOutDate;
+      return;
+   }
+   if (getparam_bool("squidguard_ignore_date",buf,&RedirectorFilterOutDate)>0) {
+      debuga(_("squidguard_ignore_date is deprecated and has been replaced by redirector_filter_out_date that does the action implied by its name as opposed to squidguard_ignore_date. Please update your configuration file.\n"));
+      RedirectorFilterOutDate=!RedirectorFilterOutDate;
+      return;
+   }
 
-   if (getparam_bool("dansguardian_ignore_date",buf,&DansguardianIgnoreDate)>0) return;
+   if (getparam_bool("dansguardian_filter_out_date",buf,&DansguardianFilterOutDate)>0) return;
+   if (getparam_bool("dansguardian_ignore_date",buf,&DansguardianFilterOutDate)>0) {
+      debuga(_("dansguardian_ignore_date is deprecated and has been replaced by dansguardian_filter_out_date that does the action implied by its name as opposed to dansguardian_ignore_date. Please update your configuration file.\n"));
+      DansguardianFilterOutDate=!DansguardianFilterOutDate;
+      return;
+   }
 
    if (getparam_string("ulimit",buf,Ulimit,sizeof(Ulimit))>0) return;
 
-   if (getparam_string("ntlm_user_format",buf,NtlmUserFormat,sizeof(NtlmUserFormat))>0) return;
+   if (getparam_list("ntlm_user_format",SET_LIST(ntml_userformat_values),buf,&NtlmUserFormat)>0) return;
 
    if (getparam_string("realtime_types",buf,RealtimeTypes,sizeof(RealtimeTypes))>0) return;
 
-   if (getparam_string("realtime_unauthenticated_records",buf,RealtimeUnauthRec,sizeof(RealtimeUnauthRec))>0) return;
+   if (getparam_list("realtime_unauthenticated_records",SET_LIST(realtime_unauth_values),buf,&RealtimeUnauthRec)>0) return;
 
    if (getparam_int("realtime_refresh_time",buf,&realtime_refresh)>0) return;
 
    if (getparam_int("realtime_access_log_lines",buf,&realtime_access_log_lines)>0) return;
 
+   if (getparam_string("LDAPHost",buf,LDAPHost,sizeof(LDAPHost))>0) return;
+
+   if (getparam_int("LDAPPort",buf,&LDAPPort)>0) return;
+
+   if (getparam_int("LDAPProtocolVersion",buf,&LDAPProtocolVersion)>0) return;
+
+   if (getparam_string("LDAPBindDN",buf,LDAPBindDN,sizeof(LDAPBindDN))>0) return;
+
+   if (getparam_string("LDAPBindPW",buf,LDAPBindPW,sizeof(LDAPBindPW))>0) return;
+
+   if (getparam_string("LDAPBaseSearch",buf,LDAPBaseSearch,sizeof(LDAPBaseSearch))>0) return;
+
+   if (getparam_string("LDAPFilterSearch",buf,LDAPFilterSearch,sizeof(LDAPFilterSearch))>0) return;
+
+   if (getparam_string("LDAPTargetAttr",buf,LDAPTargetAttr,sizeof(LDAPTargetAttr))>0) return;
+
    if (getparam_string("graph_font",buf,GraphFont,sizeof(GraphFont))>0) return;
 
    if(strstr(buf,"squid24") != 0) {
-      squid24++;
+      squid24=true;
       return;
    }
 
    if(strstr(buf,"byte_cost") != 0) {
       getword_start(&gwarea,buf);
       if (getword_multisep(wbuf,sizeof(wbuf),&gwarea,' ')<0) {
-         printf("SARG: Maybe you have a broken record or garbage in byte_cost parameter.\n");
-         exit(1);
+         debuga(_("The \"byte_cost\" parameter of the configuration file is invalid\n"));
+         exit(EXIT_FAILURE);
       }
       cost=atol(gwarea.current);
       if (getword_multisep(wbuf,sizeof(wbuf),&gwarea,' ')<0) {
-         printf("SARG: Maybe you have a broken record or garbage in byte_cost parameter.\n");
-         exit(1);
+         debuga(_("The \"byte_cost\" parameter of the configuration file is invalid\n"));
+         exit(EXIT_FAILURE);
       }
       nocost=my_atoll(gwarea.current);
       return;
    }
 
-   printf("SARG: %s %s\n",text[140],buf);
+   printf(_("SARG: Unknown option %s\n"),buf);
 }
 
 void getconf(void)
@@ -465,11 +678,11 @@ void getconf(void)
    char buf[MAXLEN];
 
    if(debug)
-      debuga("Loading configuration from: %s",ConfigFile);
+      debuga(_("Loading configuration from %s\n"),ConfigFile);
 
    if ((fp_in = fopen(ConfigFile, "r")) == NULL) {
-      fprintf(stderr, "SARG: (getconf) Cannot open file: %s\n",ConfigFile);
-      exit(1);
+      debuga(_("(getconf) Cannot open file %s\n"),ConfigFile);
+      exit(EXIT_FAILURE);
    }
 
    while (fgets(buf, sizeof(buf), fp_in) != NULL) {
@@ -483,6 +696,5 @@ void getconf(void)
    }
 
    fclose(fp_in);
-   language_load(language);
    return;
 }
