@@ -1,5 +1,6 @@
 #include "config.h"
 #include "info.h"
+#include "btree_cache.h"
 
 #ifdef HAVE_WINDOWS_H
 #include <windows.h>
@@ -62,10 +63,8 @@
 #include <sys/wait.h>
 #endif
 #ifdef HAVE_GD_H
+#define HAVE_GD 1
 #include <gd.h>
-#define HAVE_GD
-gdImagePtr im;
-gdPoint points[4];
 #endif
 #ifdef HAVE_GDFONTL_H
 #include <gdfontl.h>
@@ -94,12 +93,25 @@ gdPoint points[4];
 #ifdef HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
+#ifdef HAVE_MATH_H
+#include <math.h>
+#endif
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
 #ifdef HAVE_EXECINFO_H
 #include <execinfo.h>
 #endif
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h>
+#endif
+#ifdef HAVE_STDBOOL_H
+#include <stdbool.h>
+#else
+typedef int bool;
+#define true 1
+#define false 0
+#endif
 
 #if defined(HAVE_FOPEN64)
 #define _FILE_OFFSET_BITS 64
@@ -117,6 +129,22 @@ int mkstemps(char *template, int suffixlen);
 #define mkstemp(template) mkstemps(template,0)
 #endif
 
+#if defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
+#include <libintl.h>
+#define _(String) gettext(String)
+#ifdef gettext_noop
+#define N_(String) gettext_noop(String)
+#else
+#define N_(String) (String)
+#endif
+#else /* No NLS */
+#define _(String) (String)
+#define N_(String) (String)
+#define ngettext(String,Strings,num) (Strings)
+#define textdomain(String)
+#define bindtextdomain(Domain,Directory)
+#endif //NLS
+
 #if defined(__MINGW32__)
 #define mkdir(p,m) _mkdir(p)
 
@@ -130,62 +158,136 @@ int mkstemps(char *template, int suffixlen);
 
 #endif /*__MINGW32__*/
 
-#define MAXLEN 20000
-#define MAXLOGS 255
+#ifndef __GNUC__
+#  define  __attribute__(x)
+#endif
 
-FILE *fp_tt;
+#define MAXLEN 20000
+#define MAX_URL_LEN 40000
+#define MAX_TRUNCATED_URL 512
+#define MAX_USER_LEN 256
+#define MAX_USER_FNAME_LEN 128
+#define MAX_IP_LEN 64
+#define MAX_DATETIME_LEN 32
+#define MAXLOGS 255
+#define MAX_LOG_FILELEN 1024
+#define MAX_REDIRECTOR_LOGS 64
+#define MAX_REDIRECTOR_FILELEN 1024
+#define MAX_DATETIME_DAYS 90
+
+#define REPORT_TYPE_USERS_SITES         0x0001UL
+#define REPORT_TYPE_SITE_USER_TIME_DATE 0x0002UL
+#define REPORT_TYPE_TOPUSERS            0x0004UL
+#define REPORT_TYPE_TOPSITES            0x0008UL
+#define REPORT_TYPE_SITES_USERS         0x0010UL
+#define REPORT_TYPE_DATE_TIME           0x0020UL
+#define REPORT_TYPE_DENIED              0x0040UL
+#define REPORT_TYPE_AUTH_FAILURES       0x0080UL
+#define REPORT_TYPE_DOWNLOADS           0x0100UL
+
+#define DATA_FIELD_USER      0x0001UL
+#define DATA_FIELD_DATE      0x0002UL
+#define DATA_FIELD_TIME      0x0004UL
+#define DATA_FIELD_URL       0x0008UL
+#define DATA_FIELD_CONNECT   0x0010UL
+#define DATA_FIELD_BYTES     0x0020UL
+#define DATA_FIELD_IN_CACHE  0x0040UL
+#define DATA_FIELD_OUT_CACHE 0x0080UL
+#define DATA_FIELD_ELAPSED   0x0100UL
+
+#define TOPUSERFIELDS_NUM          0x0001UL
+#define TOPUSERFIELDS_DATE_TIME    0x0002UL
+#define TOPUSERFIELDS_USERID       0x0004UL
+#define TOPUSERFIELDS_CONNECT      0x0008UL
+#define TOPUSERFIELDS_BYTES        0x0010UL
+#define TOPUSERFIELDS_SETYB        0x0020UL
+#define TOPUSERFIELDS_IN_CACHE_OUT 0x0040UL
+#define TOPUSERFIELDS_USED_TIME    0x0080UL
+#define TOPUSERFIELDS_MILISEC      0x0100UL
+#define TOPUSERFIELDS_PTIME        0x0200UL
+#define TOPUSERFIELDS_TOTAL        0x0400UL
+#define TOPUSERFIELDS_AVERAGE      0x0800UL
+
+#define USERREPORTFIELDS_CONNECT      0x0001UL
+#define USERREPORTFIELDS_BYTES        0x0002UL
+#define USERREPORTFIELDS_SETYB        0x0004UL
+#define USERREPORTFIELDS_IN_CACHE_OUT 0x0008UL
+#define USERREPORTFIELDS_USED_TIME    0x0010UL
+#define USERREPORTFIELDS_MILISEC      0x0020UL
+#define USERREPORTFIELDS_PTIME        0x0040UL
+#define USERREPORTFIELDS_TOTAL        0x0080UL
+#define USERREPORTFIELDS_AVERAGE      0x0100UL
+
+#define INDEX_YES  0x0001UL
+#define INDEX_NO   0x0002UL
+#define INDEX_ONLY 0x0004UL
+
+#define INDEX_TREE_DATE 0x0001UL
+#define INDEX_TREE_FILE 0x0002UL
+
+#define NTLMUSERFORMAT_USER       0x0001UL
+#define NTLMUSERFORMAT_DOMAINUSER 0x0002UL
+
+#define RECORDWITHOUTUSER_IP        0x0001UL
+#define RECORDWITHOUTUSER_IGNORE    0x0002UL
+#define RECORDWITHOUTUSER_EVERYBODY 0x0004UL
+
+#define DATAFILEURL_IP   0x0001UL
+#define DATAFILEURL_NAME 0x0002UL
+
+#define DISPLAY_BYTES 0x0001UL
+#define DISPLAY_ABBREV 0x0002UL
+
+#define DATETIME_ELAP 0x0001UL
+#define DATETIME_BYTE 0x0002UL
+
+#define REALTIME_UNAUTH_REC_SHOW 0x0001UL
+#define REALTIME_UNAUTH_REC_IGNORE 0x0001UL
+
+struct periodstruct
+{
+   //! The first date of the period.
+   struct tm start;
+   //! The last date of the period.
+   struct tm end;
+   //! The textual representation of the date.
+   char text[40];
+   //! The HTML representation of the date.
+   char html[40];
+};
 
 char outdir[MAXLEN];
-char dirname[MAXLEN];
-char buf[MAXLEN];
-char user[MAXLEN];
-char period[MAXLEN];
-char msg[1024];
+char outdirname[MAXLEN];
+struct periodstruct period;
 char code[MAXLEN];
 char code2[MAXLEN];
 char tmp[MAXLEN];
-char tmp3[MAXLEN];
-char tmp4[MAXLEN];
-char tmp5[MAXLEN];
-char tmp6[MAXLEN];
-char tmp7[MAXLEN];
 char parse_out[MAXLEN];
 char arqtt[MAXLEN];
 char html[MAXLEN];
-char datestimes[MAXLEN];
 char ConfigFile[MAXLEN];
-char href[MAXLEN];
-char href2[MAXLEN];
-char href3[MAXLEN];
 char df[20];
-char day[3], month[4], year[5];
-char ltext110[50];
-char cdfrom[30];
-char cduntil[30];
 int LastLog;
-int RemoveTempFiles;
+bool RemoveTempFiles;
 char ReplaceIndex[256];
-char Index[20];
-int OverwriteReport;
-char u2[255];
-char RecordsWithoutUser[20];
-int UseComma;
+unsigned long int Index;
+bool OverwriteReport;
+unsigned long int RecordsWithoutUser;
+bool UseComma;
 char MailUtility[PATH_MAX];
 int TopSitesNum;
 int TopUsersNum;
 char ExcludeCodes[256];
 char TopsitesSortField[15];
 char TopsitesSortType[20];
-char ReportType[255];
+unsigned long int ReportType;
 char UserTabFile[255];
 char warea[MAXLEN];
 char name[MAXLEN];
-int LongUrl;
-int Ip2Name;
-char language[255];
-char bufy[MAXLEN];
+bool LongUrl;
+bool Ip2Name;
 int NAccessLog;
-char AccessLog[MAXLOGS][MAXLEN];
+char AccessLog[MAXLOGS][MAX_LOG_FILELEN];
 int AccessLogFromCmdLine;
 char Title[MAXLEN];
 char BgColor[MAXLEN];
@@ -217,68 +319,51 @@ char ExcludeUsers[255];
 char DateFormat[2];
 char PerUserLimitFile[255];
 int PerUserLimit;
-int UserIp;
+bool UserIp;
 char MaxElapsed[255];
-char datetimeby[10];
-char csort[255];
+unsigned long int datetimeby;
 char CharSet[255];
 char UserInvalidChar[255];
-int Graphs;
+bool Graphs;
 char GraphDaysBytesBarColor[255];
-int Privacy;
+bool Privacy;
 char PrivacyString[255];
 char PrivacyStringColor[30];
 char IncludeUsers[MAXLEN];
 char ExcludeString[MAXLEN];
-int SuccessfulMsg;
-char TopUserFields[255];
-char UserReportFields[255];
+bool SuccessfulMsg;
+unsigned long int TopUserFields;
+unsigned long int UserReportFields;
 char DataFile[MAXLEN];
 char DataFileDelimiter[3];
-char DataFileFields[MAXLEN];
-char DataFileUrl[20];
-int ShowReadStatistics;
+unsigned long int DataFileFields;
+unsigned long int DataFileUrl;
+bool ShowReadStatistics;
 char IndexSortOrder[5];
 char DansGuardianConf[MAXLEN];
-int DansguardianIgnoreDate;
+bool DansguardianFilterOutDate;
 char SquidGuardConf[MAXLEN];
 char SquidGuarddbHome[255];
-char SquidGuardLogFormat[MAXLEN];
-char SquidGuardLogAlternate[MAXLEN];
-int SquidguardIgnoreDate;
-int ShowSargInfo;
-int BytesInSitesUsersReport;
-int ShowSargLogo;
+char RedirectorLogFormat[4096];
+int NRedirectorLogs;
+char RedirectorLogs[MAX_REDIRECTOR_LOGS][MAX_REDIRECTOR_FILELEN];
+int RedirectorLogFromCmdLine;
+bool RedirectorFilterOutDate;
+bool ShowSargInfo;
+bool BytesInSitesUsersReport;
+bool ShowSargLogo;
 char ParsedOutputLog[MAXLEN];
 char ParsedOutputLogCompress[512];
-char DisplayedValues[20];
+unsigned long int DisplayedValues;
 char HeaderFontSize[5];
 char TitleFontSize[5];
 char wwwDocumentRoot[MAXLEN];
 char ExternalCSSFile[MAXLEN];
 char BlockIt[255];
-char BlockImage[512];
-char NtlmUserFormat[30];
-char hbc1[30];
-char hbc2[255];
-char hbc3[30];
-char hbc4[30];
-char hbc5[30];
-char hbc6[30];
-char hbc7[30];
-char hbc8[30];
-char hbc9[30];
-char hbc10[30];
-char IndexTree[10];
-int UserAuthentication;
-char AuthUserFile[255];
-char AuthName[512];
-char AuthType[255];
-char Require[512];
-char *userfile;
-char *str;
-char *str2;
-char text[200][255];
+unsigned long int NtlmUserFormat;
+unsigned long int IndexTree;
+bool UserAuthentication;
+char AuthUserTemplateFile[1024];
 char val1[MAXLEN];
 char val2[MAXLEN];
 char val3[MAXLEN];
@@ -294,13 +379,10 @@ char wwork1[MAXLEN];
 char wwork2[MAXLEN];
 char wwork3[MAXLEN];
 char mask[MAXLEN];
-char httplink[MAXLEN];
-char html_old[MAXLEN];
 char site[MAXLEN];
 char us[50];
 char email[MAXLEN];
 char test[1];
-char ouser2[255];
 char user2[MAXLEN];
 char wentp[512];
 char addr[MAXLEN];
@@ -308,9 +390,15 @@ char Ulimit[6];
 char RealtimeTypes[1024];
 char cmd[255];
 char ImageFile[255];
-char tbuf[128];
-char ip[25];
-char RealtimeUnauthRec[15];
+unsigned long int RealtimeUnauthRec;
+char LDAPHost[255];
+char LDAPBindDN[512];
+char LDAPBindPW[255];
+int LDAPPort;
+int LDAPProtocolVersion;
+char LDAPBaseSearch[255];
+char LDAPFilterSearch[512];
+char LDAPTargetAttr[64];
 char GraphFont[MAXLEN];
 
 int  idate;
@@ -320,21 +408,17 @@ int  download_count;
 int  authfail_count;
 int  dansguardian_count;
 int  squidguard_count;
+int  useragent_count;
 int  limit_flag;
-int  color1;
-int  color2;
-int  color3;
 int  z1, z2, z3;
 int  ttopen;
-int  ind2;
 int  sarglog;
 int  isalog;
 int  dfrom;
 int  duntil;
 int  dataonly;
 int  indexonly;
-int  iprel;
-int  userip;
+bool  iprel;
 int  langcode;
 int  debug;
 int  debugz;
@@ -346,18 +430,15 @@ int  SiteUsersReportLimit;
 int  DansGuardianReportLimit;
 int  SquidGuardReportLimit;
 int  UserReportLimit;
-int  dotinuser;
 int  realtime_refresh;
 int  realtime_access_log_lines;
-int  realt;
 int  rc;
 int  ntopsites;
 int  nsitesusers;
 int  nrepday;
 int  ndownload;
 int  ntopuser;
-int  squid24;
-float perc;
+bool  squid24;
 
 long long int nocost;
 float cost;
