@@ -33,14 +33,22 @@ void index_only(const char *dirname,int debug)
    DIR *dirp;
    struct dirent *direntp;
    char remove[MAXLEN];
-  
-   dirp = opendir(dirname);
+
+   if ((dirp = opendir(dirname)) == NULL) {
+      debuga(_("Failed to open directory %s - %s\n"),dirname,strerror(errno));
+      exit(EXIT_FAILURE);
+   }
    while ( (direntp = readdir( dirp )) != NULL ){
       if(strcmp(direntp->d_name,".") == 0 || strcmp(direntp->d_name,"..") == 0 || strcmp(direntp->d_name, "index.html") == 0)
          continue;
-       
-      sprintf(remove,"%s/%s",dirname,direntp->d_name);
-      unlink(remove);
+
+      if (snprintf(remove,sizeof(remove),"%s/%s",dirname,direntp->d_name)>=sizeof(remove)) {
+         debuga(_("Name of the file to remove is too long: %s/%s\n"),dirname,direntp->d_name);
+         continue;
+      }
+      if (unlink(remove) == -1) {
+         debuga(_("Failed to remove the file %s\n"),remove);
+      }
    }
 
    (void)closedir( dirp );
