@@ -33,6 +33,7 @@ void mklastlog(const char *outdir)
    FILE *fp_in, *fp_ou;
    DIR *dirp;
    struct dirent *direntp;
+   char buf[MAXLEN];
    char temp[MAXLEN];
    char warea[MAXLEN];
    char ftime[128];
@@ -46,19 +47,25 @@ void mklastlog(const char *outdir)
    if(LastLog <= 0)
       return;
 
-   sprintf(temp,"%slastlog1",outdir);
+   snprintf(temp,sizeof(temp),"%slastlog1",outdir);
    if((fp_ou=fopen(temp,"w"))==NULL) {
-     fprintf(stderr, "SARG: (lastlog) %s: %s\n",text[9],temp);
-     exit(1);
+     debuga(_("(lastlog) Cannot open temporary file %s\n"),temp);
+     exit(EXIT_FAILURE);
    }
 
-   dirp = opendir(outdir);
+   if ((dirp = opendir(outdir)) == NULL) {
+      debuga(_("Failed to open directory %s - %s\n"),outdir,strerror(errno));
+      exit(EXIT_FAILURE);
+   }
    while ((direntp = readdir( dirp )) != NULL ){
       if(strchr(direntp->d_name,'-') == 0)
          continue;
 
-      sprintf(warea,"%s%s",outdir,direntp->d_name);
-      stat(warea,&statb);
+      snprintf(warea,sizeof(warea),"%s%s",outdir,direntp->d_name);
+      if (stat(warea,&statb) == -1) {
+         debuga(_("Failed to get the creation time of %s\n"),warea);
+         continue;
+      }
       t=statb.st_ctime;
       local = localtime(&t);
       strftime(ftime, sizeof(ftime), "%Y%m%d%H%M%S", local);
@@ -66,21 +73,21 @@ void mklastlog(const char *outdir)
       ftot++;
    }
 
-   (void)closedir( dirp );
+   closedir( dirp );
    fclose(fp_ou);
 
-   sprintf(buf,"sort -n -k 1,1 -o \"%slastlog\" \"%s\"",outdir,temp);
+   snprintf(buf,sizeof(buf),"sort -n -k 1,1 -o \"%slastlog\" \"%s\"",outdir,temp);
    cstatus=system(buf);
    if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
-      fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
-      fprintf(stderr, "SARG: sort command: %s\n",buf);
-      exit(1);
+      debuga(_("sort command return status %d\n"),WEXITSTATUS(cstatus));
+      debuga(_("sort command: %s\n"),buf);
+      exit(EXIT_FAILURE);
    }
 
    unlink(temp);
 
    if(ftot<=LastLog) {
-      sprintf(temp,"%slastlog",outdir);
+      snprintf(temp,sizeof(temp),"%slastlog",outdir);
       if(access(temp, R_OK) == 0)
          unlink(temp);
       return;
@@ -88,33 +95,35 @@ void mklastlog(const char *outdir)
 
    ftot-=LastLog;
 
-   sprintf(temp,"%slastlog",outdir);
+   snprintf(temp,sizeof(temp),"%slastlog",outdir);
    if((fp_in=fopen(temp,"r"))==NULL) {
-     fprintf(stderr, "SARG: (lastlog) %s: %s\n",text[9],temp);
-     exit(1);
+     debuga(_("(lastlog) Cannot open temporary file %s\n"),temp);
+     exit(EXIT_FAILURE);
    }
 
    while(ftot>0 && fgets(buf,sizeof(buf),fp_in)!=NULL) {
       fixendofline(buf);
       getword_start(&gwarea,buf);
       if (getword(warea,sizeof(warea),&gwarea,'\t')<0) {
-         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",temp);
-         exit(1);
+         debuga(_("Maybe you have a broken record or garbage in your %s file\n"),temp);
+         exit(EXIT_FAILURE);
       }
 
       if(debug)
-         debuga("%s: %s",text[81],gwarea.current);
+         debuga(_("Removing old report file %s\n"),gwarea.current);
       if (snprintf(temp,sizeof(temp),"%s%s",outdir,gwarea.current)>=sizeof(temp)) {
-         fprintf(stderr,"SARG: Directory name too long: %s%s\n",outdir,gwarea.current);
-         exit(1);
+         debuga(_("Directory name too long: %s%s\n"),outdir,gwarea.current);
+         exit(EXIT_FAILURE);
       }
       unlinkdir(temp,0);
       ftot--;
    }
 
    fclose(fp_in);
-   sprintf(temp,"%slastlog",outdir);
-   unlink(temp);
+   snprintf(temp,sizeof(temp),"%slastlog",outdir);
+   if (unlink(temp) == -1) {
+      debuga(_("Failed to delete the file %s\n"),temp);
+   }
 
    return;
 }
