@@ -35,37 +35,39 @@ void tmpsort(void)
    int cstatus;
    const char tmpext[]=".utmp";
    int dlen;
-
    char csort[MAXLEN];
    char arqou[MAXLEN], arqin[MAXLEN], wnome[MAXLEN];
-   char field1[10]="2,2";
-   char field2[10]="1,1";
-   char field3[10]="3,3";
-   char order[4]="-r";
+   const char *field1="2,2";
+   const char *field2="1,1";
+   const char *field3="3,3";
+   const char *order="-r";
 
    if(indexonly) return;
-   if(strstr(ReportType,"users_sites") == 0) return;
+   if((ReportType & REPORT_TYPE_USERS_SITES) == 0) return;
 
    strup(UserSortField);
    if(strcmp(UserSortField,"CONNECT") == 0) {
-      strcpy(field1,"1,1");
-      strcpy(field2,"2,2");
-      strcpy(field3,"3,3");
+      field1="1,1";
+      field2="2,2";
+      field3="3,3";
    } else if(strcmp(UserSortField,"SITE") == 0) {
-      strcpy(field1,"3,3");
-      strcpy(field2,"2,2");
-      strcpy(field3,"1,1");
+      field1="3,3";
+      field2="2,2";
+      field3="1,1";
    } else if(strcmp(UserSortField,"TIME") == 0) {
-      strcpy(field1,"5,5");
-      strcpy(field2,"2,2");
-      strcpy(field3,"1,1");
+      field1="5,5";
+      field2="2,2";
+      field3="1,1";
    }
 
    strlow(UserSortOrder);
    if(strcmp(UserSortOrder,"normal") == 0)
-      order[0]='\0';
+      order="";
 
-   dirp = opendir(tmp);
+   if ((dirp = opendir(tmp)) == NULL) {
+      debuga(_("Failed to open directory %s - %s\n"),tmp,strerror(errno));
+      exit(EXIT_FAILURE);
+   }
    while ((direntp = readdir( dirp )) != NULL ){
       dlen=strlen(direntp->d_name)-(sizeof(tmpext)-1);
       if (dlen<0) continue;
@@ -87,16 +89,16 @@ void tmpsort(void)
       strcat(arqin,direntp->d_name);
 
       if(debug) {
-         debuga("%s: %s",text[54],arqou);
+         debuga(_("Sorting file: %s\n"),arqou);
       }
 
       strcat(arqou,".txt");
       sprintf(csort,"sort -n -T \"%s\" %s -k %s -k %s -k %s -o \"%s\" \"%s\"",TempDir,order,field1,field2,field3,arqou,arqin);
       cstatus=system(csort);
       if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
-         fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
-         fprintf(stderr, "SARG: sort command: %s\n",csort);
-         exit(1);
+         debuga(_("sort command return status %d\n"),WEXITSTATUS(cstatus));
+         debuga(_("sort command: %s\n"),csort);
+         exit(EXIT_FAILURE);
       }
       unlink(arqin);
 
@@ -115,16 +117,20 @@ void sort_users_log(const char *tmp, int debug)
    char user[MAXLEN];
    char wdname[MAXLEN];
    int cstatus;
-   const char unsortext[]=".unsort";
    int dlen;
+   int clen;
+   const char unsortext[]=".unsort";
 
    if(debug) {
-      debuga("%s",text[138]);
+      debuga(_("pre-sorting files\n"));
    }
 
-   sprintf(wtmp,"%s/sarg",tmp);
+   snprintf(wtmp,sizeof(wtmp),"%s/sarg",tmp);
 
-   dirp = opendir(wtmp);
+   if ((dirp = opendir(wtmp)) == NULL) {
+      debuga(_("Failed to open directory %s - %s\n"),wtmp,strerror(errno));
+      exit(EXIT_FAILURE);
+   }
    while ( (direntp = readdir( dirp )) != NULL ){
       dlen=strlen(direntp->d_name)-(sizeof(unsortext)-1);
       if (dlen<0) continue;
@@ -138,28 +144,30 @@ void sort_users_log(const char *tmp, int debug)
          strncpy(user,direntp->d_name,dlen);
          user[dlen]=0;
       } else {
-         bzero(user, MAXLEN);
+         user[0]='\0';
       }
 
       if(strcmp(direntp->d_name,"download.unsort") == 0)
-         sprintf(csort,"sort -T \"%s\" -k 3,3 -k 1,1 -k 2,2 -k 5,5 -o \"%s/%s.log\" \"%s/%s.unsort\"",
+         clen=snprintf(csort,sizeof(csort),"sort -T \"%s\" -k 3,3 -k 1,1 -k 2,2 -k 5,5 -o \"%s/%s.log\" \"%s/%s.unsort\"",
               tmp, wtmp, user, wtmp, user);
       else
-         sprintf(csort,"sort -T \"%s\" -k 5,5 -k 1,1 -k 2,2 -o \"%s/%s.log\" \"%s/%s.unsort\"",
+         clen=snprintf(csort,sizeof(csort),"sort -T \"%s\" -k 4,4 -k 1,1 -k 2,2 -o \"%s/%s.log\" \"%s/%s.unsort\"",
               tmp, wtmp, user, wtmp, user);
+      if (clen>=sizeof(csort)) {
+         debuga(_("user name too long to sort %s\n"),csort);
+         exit(EXIT_FAILURE);
+      }
       cstatus=system(csort);
       if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
-         fprintf(stderr, "SARG: sort command return status %d\n",WEXITSTATUS(cstatus));
-         fprintf(stderr, "SARG: sort command: %s\n",csort);
-         exit(1);
+         debuga(_("sort command return status %d\n"),WEXITSTATUS(cstatus));
+         debuga(_("sort command: %s\n"),csort);
+         exit(EXIT_FAILURE);
       }
       if (snprintf(wdname,sizeof(wdname),"%s/%s.unsort",wtmp,user)>=sizeof(wdname)) {
-         fprintf(stderr, "SARG: user name too long for: %s/%s.unsort\n",wtmp,user);
-         exit(1);
+         debuga(_("user name too long for %s/%s.unsort\n"),wtmp,user);
+         exit(EXIT_FAILURE);
       }
       unlink(wdname);
-      bzero(user, MAXLEN);
-
    }
    (void)closedir( dirp );
 
